from common.util.sql_util import get_insert_columns_and_values, transfer_database_name_to_hump, \
    transfer_hump_to_database_name, generate_select_sql, generate_insert_sql, generate_batch_insert_sql, \
    generate_common_query_sql, generate_create_table_sql


class TestSqlUtil:
    def test_transfer_database_name_to_hump(self):
        assert transfer_database_name_to_hump("nc_add_time") == 'ncAddTime'
        assert transfer_database_name_to_hump("nc_admission_record") == 'ncAdmissionRecord'

    def test_transfer_hump_to_database_name(self):
        assert transfer_hump_to_database_name("ncAddTime") == 'nc_add_time'
        assert transfer_hump_to_database_name("ncAdmissionRecord") == 'nc_admission_record'

    def test_generate_select_sql(self):
        print(generate_select_sql('ncpcs_all', 'nc_pathology_info'))

    def test_generate_insert_sql(self):
        print(generate_insert_sql('ncpcs_nlp', 'nc_knowledge_node'))

    def test_generate_batch_insert_sql(self):
        print(generate_batch_insert_sql('ncpcs_tool', 'nc_report_medical_info'))

    def test_generate_common_query_sql(self):
        print(generate_common_query_sql('ncpcs_tool', 'nc_report_medical_info'))

    def test_get_insert_columns_and_values(self):
        columns, values = get_insert_columns_and_values({
            "a": "深爱",
            "b": ['1', '2', '3'],
            "c": 5
        })
        assert columns == 'a,b,c'
        print(values)
        assert values == "'深爱','1,2,3',5"

    def test_generate_create_table_sql(self):
        # 测试数据
        table_name = "example_table"
        columns = {
            "id": "INT AUTO_INCREMENT PRIMARY KEY",
            "nc_add_account": "VARCHAR(50) COMMENT '添加者'",
            "nc_upd_account": "VARCHAR(50) COMMENT '添加者'",
            "nc_add_time": "VARCHAR(18) COMMENT '添加时间'",
            "nc_upd_time": "VARCHAR(18) COMMENT '更新时间'",
            "nc_medical_institution_code": "VARCHAR(30) COMMENT '组织机构代码'",
            "nc_medical_record_no": "VARCHAR(50) COMMENT '病案号'",
            "nc_discharge_time": "DATETIME COMMENT '出院时间'"
        }

        # 生成CREATE TABLE语句
        create_table_sql = generate_create_table_sql(table_name, columns)
        print(create_table_sql)






from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from base64 import b64encode


def encrypt_aes(key, plaintext):
    backend = default_backend()

    # 使用PKCS7填充
    padder = padding.PKCS7(algorithms.AES.block_size).padder()
    padded_plaintext = padder.update(plaintext) + padder.finalize()

    # 使用AES加密
    cipher = Cipher(algorithms.AES(key), modes.ECB(), backend=backend)
    encryptor = cipher.encryptor()
    ciphertext = encryptor.update(padded_plaintext) + encryptor.finalize()

    return b64encode(ciphertext)


def decrypt_aes(key, ciphertext):
    backend = default_backend()

    # 使用AES解密
    cipher = Cipher(algorithms.AES(key), modes.ECB(), backend=backend)
    decryptor = cipher.decryptor()
    decrypted_data = decryptor.update(ciphertext) + decryptor.finalize()

    # 去除填充
    unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
    unpadded_data = unpadder.update(decrypted_data) + unpadder.finalize()

    return unpadded_data




import re


def clean(phone):
    if not phone:
        return None
    pattern = r"\d{11,}"
    match = re.search(pattern, phone)
    phone = None
    if match and len(match.group()) == 11:
        phone = match.group()
    return phone



import requests

def post_url(url, data):
    # 发送POST请求，json参数会自动将字典转换为JSON格式的字符串
    response = requests.post(url, json=data)

    # 检查请求是否成功
    if response.status_code == 200:
        print('请求成功！')
    else:
        print('请求失败，状态码：', response.status_code)

    return response.json()




from common.constants.database_config import STORE_BATCH_COUNT
from common.util.database_connection_util import get_connection_by_schema


COMMON_WHERE_FORMAT = '''
<sql id="Common_Where">
    <where>
        {}    
    </where>
</sql>
'''
INSERT_SQL_FORMAT = "insert into {}({}) values({})"
SELECT_SQL_FORMAT = "select {} from {}"
BATCH_INSERT_SQL_FORMAT = '''
<insert id="batchInsert">
    <if test="list.size() > 0">
        insert into {}({}) 
        values
        <foreach collection="list" item="item" separator=",">
            ({})
        </foreach>
        ;
    </if>
</insert>
'''


def update_by_batch(table_name, relation_key_list, conn):
    if not relation_key_list:
        return
    cursor = conn.cursor()
    size = len(relation_key_list)
    i = 0
    print(update_by_batch.__name__, "执行{}任务开始, 总计{}条待修改数据".format(table_name, size))
    while i < size:
        endI = min(i + STORE_BATCH_COUNT, size)
        print(update_by_batch.__name__, "startIndex: {}, endIndex: {}".format(i, endI))
        cursor.execute("update {} set nc_valid_flag = 1 where nc_key in ({})".format(
                        table_name, ','.join(["'{}'".format(nc_key) for nc_key in relation_key_list[i: endI]])))
        conn.commit()
        i += STORE_BATCH_COUNT
    cursor.close()
    print(update_by_batch.__name__, "执行{}任务结束".format(table_name))


def store_by_batch(table_name, data_list, conn):
    if not data_list:
        return
    size = len(data_list)
    i = 0
    cursor = conn.cursor()
    print(store_by_batch.__name__, "执行{}任务开始, 总计{}条待存储数据".format(table_name, size))
    while i < size:
        endI = min(i + STORE_BATCH_COUNT, size)
        print(store_by_batch.__name__, "startIndex: {}, endIndex: {}".format(i, endI))
        _batch_store_func(table_name, data_list[i: endI], cursor)
        conn.commit()
        i += STORE_BATCH_COUNT
    cursor.close()
    print(store_by_batch.__name__, "执行{}任务结束".format(table_name))


def transfer_database_name_to_hump(column_name):
    i = 0
    ans = ''
    while i < len(column_name):
        ch = column_name[i]
        if ch == '_' and i + 1 < len(column_name):
            ans = ans + column_name[i + 1].upper()
            i += 2
        else:
            ans += ch
            i += 1
    return ans


def transfer_hump_to_database_name(string):
    # schoolAddress -> school_address
    result = ""
    for ch in string:
        if ch.isupper():
            result = result + '_' + ch.lower()
        else:
            result += ch
    return result


def generate_select_sql(schema, table_name):
    col_name_list, _ = _get_col_list(schema, table_name)
    select_sql = SELECT_SQL_FORMAT.format(", ".join(col_name_list), table_name)
    return select_sql


def generate_insert_sql(schema, table_name):
    col_name_list, item_name_list = _get_col_list(schema, table_name)
    item_name_str = ", ".join(item_name_list)
    item_name_str = item_name_str.replace("[", "{")
    item_name_str = item_name_str.replace("]", "}")
    insert_sql = INSERT_SQL_FORMAT.format(table_name, ", ".join(col_name_list), item_name_str)
    return insert_sql


def generate_batch_insert_sql(schema, table_name):
    col_name_list, item_name_list = _get_col_list(schema, table_name)
    item_name_str = ",\n\t\t\t".join(item_name_list)
    item_name_str = item_name_str.replace("[", "{")
    item_name_str = item_name_str.replace("]", "}")
    batch_insert_sql = BATCH_INSERT_SQL_FORMAT.format(table_name, ",\n\t\t\t".join(col_name_list), item_name_str)
    return batch_insert_sql


def generate_common_query_sql(schema, table_name):
    OTHER_IF_SQL_FORMAT = '''
        <if test="param.{} != null">
            and {} = #[param.{}]
        </if>'''

    STR_IF_SQL_FORMAT = '''
        <if test="param.{} != null and param.{} != ''">
            and {} = #[param.{}]
        </if>'''
    CHECK_COL_SQL_FORMAT = "SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '{}' AND TABLE_SCHEMA = '{}'"
    conn = get_connection_by_schema(schema)
    cursor = conn.cursor()
    cursor.execute(CHECK_COL_SQL_FORMAT.format(table_name, schema))
    if_sql = ""
    for ele in cursor.fetchall():
        hump_name = transfer_database_name_to_hump(ele[0])
        if ele[1].find("char") != -1 or ele[1].find("text") != -1:
            if_sql = if_sql + STR_IF_SQL_FORMAT.format(hump_name, hump_name, ele[0], hump_name)
        if ele[1].find("int") != -1:
            if_sql = if_sql + OTHER_IF_SQL_FORMAT.format(hump_name, ele[0], hump_name)
        if ele[1].find("datetime") != -1:
            if_sql = if_sql + OTHER_IF_SQL_FORMAT.format(hump_name, ele[0], hump_name)

    if_sql = if_sql.replace("[", "{")
    if_sql = if_sql.replace("]", "}")
    common_where_sql = COMMON_WHERE_FORMAT.format(if_sql)
    return common_where_sql


def generate_create_table_sql(table_name, columns, charset='utf8mb4', collate='utf8mb4_unicode_ci'):
    """
    生成创建表的SQL语句

    参数:
    table_name (str): 表名
    columns (dict): 包含列名和数据类型的字典，例如 {'column1': 'INT', 'column2': 'VARCHAR(255)'}
    charset (str): 字符编码，默认为utf8
    collate (str): 校对规则，默认为utf8_general_ci

    返回:
    str: CREATE TABLE语句
    """
    sql = f"CREATE TABLE IF NOT EXISTS {table_name} ("
    for column_name, column_type in columns.items():
        sql += f"{column_name} {column_type}, "
    sql = sql[:-2]  # 移除最后的逗号和空格
    sql += f") CHARACTER SET {charset} COLLATE {collate};"
    return sql


def _get_insert_columns(columns):
    return ','.join(columns)


def _get_insert_values(values):
    """
    根据values获取insert sql
    :param values:
    :return:
    """
    ans = ''
    for value in values:
        if not value:
            ans += "'',"
        elif isinstance(value, str):
            # value = value[:20000]
            value = value.replace("'", "")
            value = value.replace("\\", "\\\\")
            ans += ("'{}',".format(value))
        elif isinstance(value, int):
            ans += ("{},".format(value))
        elif hasattr(value, '__iter__'):
            ans += "'{}',".format(','.join([v for v in value if v and isinstance(v, str)]))
    return ans[:-1]


def get_insert_columns_and_values(data):
    return _get_insert_columns(data.keys()), _get_insert_values(data.values())


def _get_col_list(schema, table_name):
    conn = get_connection_by_schema(b'my pleasure lord', schema)
    cursor = conn.cursor()
    CHECK_COL_SQL_FORMAT = "SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '{}' AND TABLE_SCHEMA = '{}'"
    ITEM_FORMAT = "#[item.{}]"
    cursor.execute(CHECK_COL_SQL_FORMAT.format(table_name, schema))
    col_name_list = []
    item_name_list = []
    for ele in cursor.fetchall():
        col_name_list.append(ele[0])
        item_name_list.append(ITEM_FORMAT.format(transfer_database_name_to_hump(ele[0])))
    return col_name_list, item_name_list


def _batch_store_func(table_name, data_list, cursor):
    insertSqlFormat = 'insert into {}({}) values{}'
    columns = ','.join(data_list[0].keys())
    value_list = []
    for data in data_list:
        value_list.append('({})'.format(_get_insert_values(data.values())))
    insertSql = insertSqlFormat.format(table_name, columns, ','.join(value_list))
    cursor.execute(insertSql)



import random
from collections import Counter
from common.constants.level_dict import COLUMN_DICT
from common.entity.relation_key import RelationKey
from common.util.string_util import split_text

ORDER_TABLE_DICT = {
    "nc_daily_disease_course": {
        "pageUuid": "606010000",
        "orderSql": " order by nc_disease_course_no ASC,sort_time ASC, nc_record_time, nc_rid"
    },
    "nc_pathology_info": {
        "pageUuid": "611010000",
        "orderSql": " order by nc_pathology_no ASC,sort_time ASC"
    },
    "nc_imageology_exam": {
        "pageUuid": "613010000",
        "orderSql": " order by nc_exam_order ASC,nc_report_no ASC,sort_time ASC"
    }
}

GET_TEXT_SQL_FORMAT = "select {} from {} where nc_medical_institution_code = '{}' and nc_medical_record_no = '{}' and "\
                     "nc_discharge_time = '{}' and nc_hedge = 0 and nc_data_status != 99"


def extract_medical_text(cursor, relation_key, column_dict=COLUMN_DICT):
    medical_text = []
    for table_name, column_list in column_dict.items():
        get_text_sql = GET_TEXT_SQL_FORMAT.format(','.join(column_list), table_name, relation_key.medical_institution_code,
                                                  relation_key.medical_record_no, relation_key.discharge_time)
        page_uuid, get_text_sql = generate_order_sql(get_text_sql)
        cursor.execute(get_text_sql)
        page = 1
        for ele in cursor.fetchall():
            medical_text.extend([
                {
                    "表名": table_name,
                    "字段名": column_list[i],
                    "文本列表": split_text(val),
                    "页码": page,
                    "组号": page_uuid
                }
                for i, val in enumerate(ele) if val]
            )
            page += 1
    return medical_text


def extract_all_relation_key(cursor):
    cursor.execute("select nc_medical_institution_code, nc_medical_record_no, nc_discharge_time from nc_medical_record_first_page where nc_data_report_type = 1")
    relation_key_list = []
    for ele in cursor.fetchall():
        relation_key_list.append(RelationKey(ele[0], ele[1], ele[2].strftime('%Y-%m-%d-%H-%M-%S')))

    return relation_key_list


def generate_order_sql(sql):
    for table_name, order_info in ORDER_TABLE_DICT.items():
        if sql.count(table_name):
            return order_info['pageUuid'], sql + order_info['orderSql']
    return '', sql


def sentence_count(sentence_list):
    count_dict = Counter(sentence_list)
    for sentence, count in sorted(count_dict.items(), key=lambda t: t[1], reverse=True):
        print(sentence, count)


def random_pick_relation_key(relation_key_dict, num_to_pick):
    selected_relation_key_list = []
    avg_to_pick = num_to_pick // len(relation_key_dict.keys())
    left_relation_key_list = []
    for _, relation_key_list in relation_key_dict.items():
        if len(relation_key_list) <= avg_to_pick:
            selected_relation_key_list.extend(relation_key_list)
        else:
            random_list = random.sample(relation_key_list, avg_to_pick)
            random_set = set(random_list)
            selected_relation_key_list.extend(random_list)
            left_relation_key_list.extend([relation_key for relation_key in relation_key_list
                                           if relation_key not in random_set])
    selected_relation_key_list.extend(random.sample(left_relation_key_list, num_to_pick - len(selected_relation_key_list)))
    return selected_relation_key_list


from common.constants.level_dict import ICCC3_CODE_NAME_DICT

from pypinyin import lazy_pinyin

from common.util.string_util import remove_special_symbols, remove_bracket


def handle_iccc3_name(name):
    name = remove_special_symbols(name)
    name = remove_bracket(name)
    return name


ICCC3_NAME_TO_CODE_DICT = {}
for code, name_list in ICCC3_CODE_NAME_DICT.items():
    for name in name_list:
        ICCC3_NAME_TO_CODE_DICT[handle_iccc3_name(name)] = code


def convert_iccc3_name_to_iccc3_code(iccc3_name):
    iccc3_name = handle_iccc3_name(iccc3_name)
    return ICCC3_NAME_TO_CODE_DICT.get(iccc3_name, None)


def convert_iccc3_name_to_iccc3_code_best(iccc3_name):
    iccc3_code = convert_iccc3_name_to_iccc3_code(iccc3_name)
    if iccc3_code:
        return iccc3_code

    iccc3_name = handle_iccc3_name(iccc3_name)
    for name, code in ICCC3_NAME_TO_CODE_DICT.items():
        if len(iccc3_name) < len(name):
            continue
        match = True
        for i, ch in enumerate(name):
            if ch != iccc3_name[i] and lazy_pinyin(ch) != lazy_pinyin(iccc3_name[i]):
                match = False
                break
        if match:
            return code

    return None


def get_large_group(iccc3_code):
    bound = iccc3_code.find(".")
    if bound == -1:
        bound = len(iccc3_code)
    return iccc3_code[: bound]


def extract_iccc3_name_large_group(iccc3_name):
    iccc3_code = convert_iccc3_name_to_iccc3_code_best(iccc3_name)
    if iccc3_code:
        return get_large_group(iccc3_code)
    return None




import concurrent.futures


def execute_by_multi_thread(tasks, max_workers=8):
    results = []
    with concurrent.futures.ThreadPoolExecutor(max_workers) as executor:
        # 提交任务给线程池，并返回 Future 对象列表
        futures = [executor.submit(*task) for task in tasks]
        # 获取任务的结果
        for future in concurrent.futures.as_completed(futures):
            try:
                results.extend(future.result())  # 阻塞等待任务完成并获取结果
            except Exception as e:
                print(f"Exception occurred: {e}")
    return results


from common.util.id_card_util import validate


class TestIdCardUtil:
    def test_validate(self):
        assert validate("111111111111111") == False
        assert validate("111111111111111111") == False
        assert validate("350301199609270718") == True


import re

BEST_LEN = 15
# 常见的单字和复姓列表
common_surnames = [
    "李", "王", "张", "刘", "陈", "杨", "黄", "赵", "周", "吴",
    "徐", "孙", "马", "朱", "胡", "林", "郭", "何", "高", "罗",
    "郑", "梁", "谢", "宋", "唐", "许", "韩", "冯", "邓", "曹",
    "彭", "曾", "肖", "田", "董", "袁", "潘", "于", "蒋", "蔡",
    "余", "杜", "叶", "程", "苏", "魏", "吕", "丁", "任", "沈",
    "姚", "卢", "姜", "崔", "钟", "谭", "陆", "汪", "范", "金",
    "石", "廖", "贾", "夏", "韦", "傅", "方", "白", "邹", "孟",
    "熊", "秦", "邱", "江", "尹", "薛", "闫", "段", "雷", "侯",
    "龙", "史", "陶", "黎", "贺", "顾", "毛", "郝", "龚", "邵",
    "万", "钱", "严", "赖", "覃", "洪", "武", "莫", "孔", "汤",
    "习", "尤", "苗", "俞", "鲍", "章", "施", "窦", "岑", "乐",
    "成", "詹", "欧阳", "司马", "端木", "上官",  # 复姓
]


def any_match(string, match_list):
    result = []
    for match in match_list:
        if string.find(match) != -1:
            result.append(match)
    return result


def any_match_bool(string, match_list):
    for match in match_list:
        if string.find(match) != -1:
            return True
    return False


def extract_digit(string):
    result = ""
    for ch in string:
        if ch.isdigit():
            result += ch
    return result


def find_all(string, substring):
    positions = []
    start = string.find(substring)

    while start != -1:
        positions.append(start)

        # 更新起始位置为当前子串后面的位置
        start += len(substring)
        next_pos = string[start:].find(substring)
        if next_pos == -1:
            break
        start = next_pos + start

    return positions

#去掉所有的空格、回车等特殊符号
def remove_special_symbols(string):
    return string.replace("\n", "").replace("\r", "").replace("\b", "").replace("\t", "").replace(" ", "").replace('', '')

def remove_bracket(string):
    return string.replace("(", "").replace(")", "").replace("（", "").replace("）", "")

def has_chinese(sentence):
    for ch in sentence:
        if is_chinese(ch):
            return True
    return False


# ---------------------------功能:判断字符是不是汉字-------------------------------
def is_chinese(char):
    if '\u4e00' <= char <= '\u9fff':
        return True
    return False


def handle_short_sentence(segment_list):
    temp_split_pos_list = []
    i = 0
    while i < len(segment_list) - 1:
        cur_seg_begin, cur_seg_end = segment_list[i]
        next_seg_begin, next_seg_end = segment_list[i+1]
        if cur_seg_end - cur_seg_begin < BEST_LEN and next_seg_end - next_seg_begin < BEST_LEN:
            temp_split_pos_list.append((cur_seg_begin, next_seg_end))
            i += 2
        else:
            temp_split_pos_list.append((cur_seg_begin, cur_seg_end))
            i += 1
    if i < len(segment_list):
        temp_split_pos_list.append(segment_list[i])
    return temp_split_pos_list


def split_text(text, pattern="。"):
    if text == '-':
        return []
    split_pos_list = []
    begin = 0
    for match in re.finditer(pattern, text):
        pos = match.start()
        split_pos_list.append((begin, pos))
        begin = pos+1
    split_pos_list.append((begin, len(text)))

    #TODO 长句处理
    #短句处理
    split_pos_list = handle_short_sentence(split_pos_list)
    split_text_list = []
    for begin, end in split_pos_list:
        if begin >= end:
            continue
        split_text_list.append(text[begin: end])
    return split_text_list


def locate_two_word(content, word1, word2, keep_order=True):
    word1_index_list = find_all(content, word1)
    word2_index_list = find_all(content, word2)

    if not word1_index_list or not word2_index_list:
        return None

    index_tuple_list = []
    for word1_index in word1_index_list:
        for word2_index in word2_index_list:
            index_tuple_list.append((word1_index, word2_index))

    index_tuple_list.sort(key=lambda ele: abs(ele[0]-ele[1]))
    if keep_order:
        for index_tuple in index_tuple_list:
            if index_tuple[0] < index_tuple[1]:
                return index_tuple
    return index_tuple_list[0]


def cut_from_back_to_front(text, length):
    if length <= len(text):
        return text[-length:]
    return text


def is_chinese_name(name):
    # 正则表达式检查全部为中文字符
    if not re.match(r'^[\u4e00-\u9fa5]+$', name):
        return False

    # 检查名字长度为2到4个汉字
    if len(name) < 2 or len(name) > 4:
        return False

    # 检查是否以常见姓氏开头
    if any(name.startswith(surname) for surname in common_surnames):
        return True

    return False
